import base64
from datetime import datetime
from google.auth.exceptions import DefaultCredentialsError
from google.oauth2 import service_account
import vertexai
from vertexai.generative_models import GenerativeModel, Part, SafetySetting, FinishReason
import vertexai.generative_models as generative_models
import sys
import json

def generate(prompt):
    key_path = "/var/www/html/smart-hub/public/futurefous-b541452ac5cf.json"
    try:
        credentials = service_account.Credentials.from_service_account_file(
            key_path,
            scopes=["https://www.googleapis.com/auth/cloud-platform"]
        )
        vertexai.init(project="futurefous", location="europe-west1", credentials=credentials)
        
        model = GenerativeModel("gemini-1.5-flash-002")
        
    	# prompt = "text": f"{text}"
        full_prompt = ' '.join(prompt)

        generation_config = {
            "max_output_tokens": 1000,
            "temperature": 0,
            "top_p": 0.95,
        }
        safety_settings = [
            SafetySetting(
                category=SafetySetting.HarmCategory.HARM_CATEGORY_HATE_SPEECH,
                threshold=SafetySetting.HarmBlockThreshold.BLOCK_ONLY_HIGH
            ),
            SafetySetting(
                category=SafetySetting.HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT,
                threshold=SafetySetting.HarmBlockThreshold.BLOCK_ONLY_HIGH
            ),
            SafetySetting(
                category=SafetySetting.HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT,
                threshold=SafetySetting.HarmBlockThreshold.BLOCK_ONLY_HIGH
            ),
            SafetySetting(
                category=SafetySetting.HarmCategory.HARM_CATEGORY_HARASSMENT,
                threshold=SafetySetting.HarmBlockThreshold.BLOCK_ONLY_HIGH
            ),
        ]
    
        responses = model.generate_content(
            full_prompt,
            generation_config=generation_config,
            safety_settings=safety_settings,
        )

        query_result = responses.text.strip()
        return query_result

    except Exception as e:
        return f"An error occurred during SQL generation: {str(e)}"


if __name__ == "__main__":
    if len(sys.argv) > 1:
        text = sys.argv[1]
        question = text.replace('\\"', '')

        # prompt = f"""
        #     Only generate SQL queries for valid, coherent questions. Do not generate queries for gibberish or nonsensical inputs, including those that consist solely of special characters, or lack meaningful content.
        #     Do not generate queries for gibberish or nonsensical inputs, including those that consist solely of any numbers, or lack meaningful content.

        #     Generate a SQL query to answer the following question: {question}
        #     Table 1 - referral_split_album
        #     Columns 1 - rsid (integer), upc (integer), user_percent (integer), referrer_uid (integer), percent (integer)

        #     Table 2 - users
        #     Columns 2 - uid (integer), name (string), mail (string)

        #     Table 3 - comm_form_status
        #     Columns 3 - id (integer) , type (string), form_id (integer), status (string), submitted_at (date), closed_by (string)
        #     - type values are account or revenue or content or oac
        #     - status values are approve or reject or request

        #     Table 4 - discography_api_report
        #     Columns 4 - id (integer) , upc (integer), release_status (string), main_dis_option (string)

        #     Table 5 - comm_staffs
        #     Columns 5 - uid (integer) , name (string) , mail (string)

        #     Table 6 - comm_tbl_smthub_ticket
        #     Columns 6 - id (integer) , uid (integer), uname (string),assigned_to(integer),subject(string),priority(string),tickets_received(datetime),completed_at(string)
       
        #     Table 7 - comm_grouplist
        #     Columns 7 - userid (integer) , me_email (string)

        #     - comm_grouplist table get the associated email address which refers column comm_grouplist.me_email using it's userid matches with users table uid .Get comm_grouplist.me_email only
        #     - comm_grouplist.groupid equals to comm_groups.id so we can check the user type


        #     Table 8 - comm_groups
        #     Columns 8 - id (integer) , groups (string)
        #     - groups values are Premium User or Priority User or HP Account ( High Profile )

        #     OTL related tables are in below

        #     Table 9 - comm_account_closure
        #     Columns 9 - form_id (integer) , userID (integer), payment_created_at (datetime)
        #     -if they ask about id which equals to form_id and userid which equals to userID

        #     Table 10 - comm_content_mapping
        #     Columns 10 - id (integer) , userid (integer)

        #     Table 11 - comm_revenue_form
        #     Columns 11 - id (integer) , userid (integer)

        #     Table 12 - comm_oac_form
        #     Columns 12 - id (integer) , userid (integer)

        #     - for otl related questions ,don't include comm_tbl_smthub_ticket table include only comm_form_status and users  
        #     - otl tickets completed list check with comm_form_status table if status is approve or reject which equals to completed .if status is request which equals to waiting for user information.
        #     - Also,comm_form_status.type defines that account which equals to comm_account_closure table,content which equals to comm_content_mapping table,revenue which equals to comm_revenue_form table and oac which equals to comm_oac_form table

        #     When asking about tickets completed pending list comm_tbl_smthub_ticket seperately and comm_form_status seperately.
        #     When asking about assignees, include all relevant details related to the assigned_to field, which corresponds to comm_staffs.uid. 
        #     Do not include priority filtering unless explicitly asked for high or critical(P4) or low(p1) priority tickets.
        #     For tickets get all details from the table comm_tbl_smthub_ticket and join with other tables
        #     For upc status don't join with other table

        #     Please follow these guidelines:
        #     1. MySQL 5.6.51, you'll need to simplify your query significantly and avoid modern constructs.

        #     Please provide only the SQL query without any additional explanation.
        # """


        # prompt = f"""
        #     Only generate SQL queries for valid, coherent questions. Do not generate queries for gibberish or nonsensical inputs, including those that consist solely of special characters, or lack meaningful content.
        #     Do not generate queries for gibberish or nonsensical inputs, including those that consist solely of any numbers, or lack meaningful content.

        #     Generate a SQL query to answer the following question: {question}
        #     Table 1 - referral_split_album
        #     Columns 1 - rsid (integer), upc (integer), user_percent (integer), referrer_uid (integer), percent (integer)

        #     Table 2 - users
        #     Columns 2 - uid (integer), name (string), mail (string)

        #     Table 3 - comm_form_status
        #     Columns 3 - id (integer) , type (string), form_id (integer), status (string), submitted_at (date), closed_by (string)
        #     - status values are approve or reject or request
        #     - type values are account or content or revenue or oac
        #     - closed_by related to comm_staffs.uid


        #     Table 4 - discography_api_report
        #     Columns 4 - id (integer) , upc (integer), release_status (string), main_dis_option (string)

        #     Table 5 - comm_staffs
        #     Columns 5 - uid (integer) , name (string) , mail (string)

        #     Table 6 - comm_tbl_smthub_ticket
        #     Columns 6 - id (integer) , uid (integer), uname (string),assigned_to(integer),subject(string),priority(string),tickets_received(datetime),completed_at(string)

        #     Table 7 - comm_grouplist
        #     Columns 7 - userid (integer) , me_email (string)

        #     - comm_grouplist table get the associated email address which refers column comm_grouplist.me_email using it's userid matches with users table uid .Get comm_grouplist.me_email only
        #     - comm_grouplist.groupid equals to comm_groups.id so we can check the user type


        #     Table 8 - comm_groups
        #     Columns 8 - id (integer) , groups (string)
        #     - groups values are Premium User or Priority User or HP Account ( High Profile )


        #     Table 9 - comm_account_closure
        #     Columns 9 - form_id (integer) , userID (integer), payment_created_at (datetime)
        #     - OTL relationship only use check table comm_account_closure another information ignore sentence.
        #     - for otl request ,don't include other tables 
        #     - if they ask about id which equals to form_id and userid which equals to userID

        #     Table 10 - comm_content_mapping
        #     Columns 10 - id (integer) , userid (integer)

        #     Table 11 - comm_revenue_form
        #     Columns 11 - id (integer) , userid (integer)

        #     Table 12 - comm_oac_form
        #     Columns 12 - id (integer) , userid (integer)


        #     Please follow these guidelines:

        #     For OTL-related questions, exclude the comm_tbl_smthub_ticket table; include only comm_form_status and users.
        #     For completed tickets, check comm_form_status for status (approve or reject) and determine request status as waiting for user information.
        #     comm_form_status.type defines the corresponding OTL table.
        #     When asking about assignees, include all relevant details related to the assigned_to field from comm_staffs.uid.
        #     Do not include priority filtering unless explicitly asked for high (P4) or low (P1) priority tickets.
        #     For ticket queries, retrieve all details from comm_tbl_smthub_ticket and join with other relevant tables.
        #     For UPC status, do not join with other tables.
        #     Note: Use MySQL 5.6.51 syntax; simplify queries and avoid modern constructs.


        #     Please provide only the SQL query without any additional explanation.
        # """
            #Do not generate queries for gibberish or nonsensical inputs, including those that consist solely of any numbers without text, or lack meaningful content.
#         prompt = f"""
                  
#     SELECT 
#     r.referrer_uid,
#     r.percent AS referral_percent,
#     u.name AS user_name,
#     u.mail AS user_email,
#     cfs.type AS form_type,
#     cfs.status AS form_status,
#     cfs.submitted_at,
#     dar.upc,
#     dar.release_status,
#     dar.main_dis_option,
#     cs.name AS staff_name,
#     cs.mail AS staff_email,
#     st.subject,
#     st.priority,
#     st.tickets_received,
#     g.groups,
#     ca.payment_created_at
# FROM 
#     referral_split_album r
# JOIN 
#     users u ON r.referrer_uid = u.uid
# JOIN 
#     comm_form_status cfs ON cfs.form_id = r.rsid
# JOIN 
#     discography_api_report dar ON dar.upc = r.upc
# JOIN 
#     comm_staffs cs ON cs.uid = r.referrer_uid
# JOIN 
#     comm_tbl_smthub_ticket st ON st.uid = u.uid
# JOIN 
#     comm_grouplist gl ON gl.userid = u.uid
# JOIN 
#     comm_groups g ON g.id = st.assigned_to
# JOIN 
#     comm_account_closure ca ON ca.form_id = cfs.form_id AND ca.userID = u.uid;


#                 Please follow these guidelines:
#                 1. MySQL 5.6.51, you'll need to simplify your query significantly and avoid modern constructs.

#                     Above is sample join query for all my tables. Please analyze above query and generate query for my question without explanation - only query
#                     Generate query for each question eventhough if this duplicate request

#         """

        # prompt = f"""
        #     SELECT 
        #         r.referrer_uid,
        #         r.percent AS referral_percent,
        #         u.name AS user_name,
        #         u.mail AS user_email,
        #         cfs.type AS form_type,
        #         cfs.status AS form_status,
        #         cfs.submitted_at,
        #         dar.upc,
        #         dar.release_status,
        #         cs.name AS staff_name,
        #         cs.mail AS staff_email,
        #         st.subject,
        #         st.priority,
        #         st.tickets_received,
        #         g.groups,
        #         ca.payment_created_at

        #     FROM 
        #         referral_split_album r
        #     JOIN 
        #         users u ON r.referrer_uid = u.uid
        #     JOIN 
        #         comm_form_status cfs ON cfs.form_id = r.rsid
        #     JOIN 
        #         discography_api_report dar ON dar.upc = r.upc
        #     JOIN 
        #         comm_staffs cs ON cs.uid = r.referrer_uid
        #     JOIN 
        #         comm_tbl_smthub_ticket st ON st.uid = u.uid
        #     JOIN 
        #         comm_grouplist gl ON gl.userid = u.uid
        #     JOIN 
        #         comm_groups g ON g.id = st.assigned_to
        #     JOIN 
        #         comm_account_closure ca ON ca.form_id = cfs.form_id AND ca.userID = u.uid;

        #     Above is sample join query for all my tables. Please analyze above query and generate query for my question without explanation - only query
        #     Generate query for each question eventhough if this duplicate request
        # """
        prompt = f"""
            Only generate SQL queries for valid, coherent questions. Do not generate queries for gibberish or nonsensical inputs, including those that consist solely of special characters.
            for this type give me output like INVALID REQUEST

            Generate a SQL query to answer the following question: {question}
            In Query only  get the column name mentioned below

            Table 1 - referral_split_album
            Columns 1 - rsid (integer), upc (integer), user_percent (integer), referrer_uid (integer), percent (integer)
            - Description: This table checks the share/distribution/split of the UPC number.

            - Sample Question : share distribution split for this 5054316798316(upc number)?

            Table 2 - users
            Columns 2 - uid (integer), name (string), mail (string)
            - Description: Check the email address registered in this table or not.generate query in the type yes or no

            Table 3 - comm_form_status
            Columns 3 - id (integer) , type (string), form_id (integer), status (string), submitted_at (date), closed_by (string)

            Table 4 - discography_api_report
            Columns 4 - id (integer) , upc (integer), release_status (string), main_dis_option (string)
            - This table checks the status of upc or status of number
            - release_status values are Approved or DELETED or Disapproved or In Review or Pending Moderation or Removed from Sale or Unfinished or Your Action Needed

            Table 5 - comm_staffs
            Columns 5 - uid (integer) , name (string) , mail (string)

            Table 6 - comm_tbl_smthub_ticket
            Columns 6 - id (integer) , uid (integer), uname (string),assigned_to(integer),subject(string),priority(string),tickets_received(datetime),completed_at(string)

            Table 7 - comm_grouplist
            Columns 7 - userid (integer) , me_email (string)
            comm_grouplist table get the only associated email address which refers column comm_grouplist.me_email using it's userid matches with users table uid .Get comm_grouplist.me_email only
            - comm_grouplist.groupid equals to comm_groups.id so we can check the user type

            Table 8 - comm_groups
            Columns 8 - id (integer) , groups (string)
            - groups values are Premium User or Priority User or HP Account ( High Profile )

            OTL related tables are in below

            Table 9 - comm_account_closure
            Columns 9 - form_id (integer) , userID (integer), payment_created_at (datetime)
            - Description: Normally matches with the form_id column; do not use id. If they ask about userID, use userID. This table is related to payment task.
            - sample query for get details of particular id: select payment_created_at from comm_account_closure where form_id= number;

            For otl request , don't include other tables.
            When asking about assignees, include all relevant details related to the assigned_to field, which corresponds to comm_staffs.uid. 
            Do not include priority filtering unless explicitly asked for high or critical(P4) or low(p1) priority tickets.
            For tickets get all details from the table comm_tbl_smthub_ticket and join with other tables
            For upc status don't join with other table

            Please follow these guidelines:
            1. MySQL 5.6.51, you'll need to simplify your query significantly and avoid modern constructs.

            If an email is associated, get it from the comm_grouplist table; otherwise, retrieve it from the users table. 
            To check if an email is registered, only use the users table without joining others.
            if it's related payment refer comm_account_closure table.Use this query for payment created time in last month : SELECT * FROM comm_account_closure WHERE payment_created_at <= DATE_SUB(NOW(), INTERVAL 1 MONTH);

            Please provide only the SQL query without any additional explanation.
        """
        generate = generate(prompt)
        print(generate)
        
    else:
        print("Please provide input text as command-line arguments.")

# how to use outstand and total balance use uid 38


# text = sys.argv[1]
# decoded_string = text.replace('\\"', '').strip()
# # print(decoded_string)
# # print(json.loads(text))
# keywords = ['hai', 'hello', 'hi', 'hellow']
# if decoded_string in keywords:
#     print("Hello! How can I assist you today?")
# else:
#     print('else')