import sys
from google.oauth2 import service_account
import vertexai
from vertexai.generative_models import GenerativeModel, Part, SafetySetting, FinishReason

def generate_sql(question):
    creds = service_account.Credentials.from_service_account_file("/var/www/html/smart-hub/public/futurefous-b541452ac5cf.json")
    vertexai.init(project="futurefous", location="europe-west1", credentials=creds)
    model = vertexai.generative_models.GenerativeModel("gemini-1.5-flash-002")
    generation_config = {
        "max_output_tokens": 8192,
        "temperature": 0,
        "top_p": 0.95,
    }

    safety_settings = [
    SafetySetting(
        category=SafetySetting.HarmCategory.HARM_CATEGORY_HATE_SPEECH,
        threshold=SafetySetting.HarmBlockThreshold.OFF
    ),
    SafetySetting(
        category=SafetySetting.HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT,
        threshold=SafetySetting.HarmBlockThreshold.OFF
    ),
    SafetySetting(
        category=SafetySetting.HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT,
        threshold=SafetySetting.HarmBlockThreshold.OFF
    ),
    SafetySetting(
        category=SafetySetting.HarmCategory.HARM_CATEGORY_HARASSMENT,
        threshold=SafetySetting.HarmBlockThreshold.OFF
    ),
    ]
    return model.generate_content(question,generation_config=generation_config,safety_settings=safety_settings).text.strip()



if __name__ == "__main__":
    if len(sys.argv) > 1:

        text = sys.argv[1]
        question = text.replace('\\"', '')

        prompt =f"""

        ONLY generate SQL queries for valid and coherent questions. If the input is nonsensical or gibberish, output: `INVALID REQUEST`.

DO NOT generate queries for the following:
- `UPDATE`, `DELETE`, `ALTER`, `ADD`, `CREATE`, or any database credentials (e.g., passwords, activation keys). Output: `INVALID REQUEST`.
- Requests for table lists, column names, or database structure details. Output: `INVALID REQUEST`.
- Non-query-related questions like greetings, thanks, or irrelevant inquiries (e.g., "Who are you?", "Tell me your purpose","how can you help me"). Output: `INVALID REQUEST`.

Donot generate query , 
SAMPLES.. If they ask how you help like that ..  Output: `INVALID REQUEST`.

DO NOT generate nonsensical or gibberish,output: `INVALID REQUEST`.

if input is INVALID ,output: `INVALID REQUEST`.

**General Guidelines:**
- Only generate `SELECT` queries.
- Adhere to MySQL 5.6.51 syntax. Avoid modern constructs (e.g., `HAVING`).
- Only generate queries, no extra text or explanation.
- Identify input numbers: 
  - **13-digit** numbers = UPC.
  - Any other number = user ID (UID).
- Include provided values in the query.

**Forms Handling:**
- For **OTL forms**, retrieve data from all four relevant forms.
- For **specific form types**, only generate queries for the mentioned form.

**Query Structure:**

1. **General Forms Query:**
   - Include details from:
     - `comm_account_closure`
     - `comm_content_mapping`
     - `comm_revenue_form`
     - `comm_oac_form`
   - JOIN with `users`, `comm_form_status`, `comm_staffs`, `comm_grouplist`, and `comm_groups`.
   - Use `GROUP BY` appropriately to prevent aggregation issues.
   - Generate the exact form-based query structure provided.

sample query for get details for all 4 forms: **If they mention any type of form particularly get that form details alone no need to union all.**

if they ask form details get all use union all,else get if they mention type of form particularly get that form details alone no need to union all.

--comm_staffs have column names uid,name,type in that type has Staff and Admin & name has Superadmin , Admin1 and Admin2.

does the payment task for the otl forms is created ?  -- payment questions get details only from comm_account_closure.payment_created_at no need use other tables.

any questions related to payments/asana payment task  get details only from comm_account_closure.payment_created_at


--account closure details--
if they ask about get the payment task created/asana payament task/does the payment task for the otl forms is created ? --> get Account closure comm_account_closure.payment_created_at


SELECT 'FORMS/POC' AS 'FORM/TICKET',
       st.id AS ticket_id,
       CASE 
           WHEN st.tag_id = 1 THEN 'account closure'
           WHEN st.tag_id = 2 THEN 'content mapping'
           WHEN st.tag_id = 4 THEN 'revenue share'
           WHEN st.tag_id = 3 THEN 'youtube OAC'
           ELSE 'Unknown Form'
       END AS form_name,
       COALESCE(f.userid, st.uid) AS userid, 
       COALESCE(f.username, st.from_name) AS username, 
       COALESCE(f.email, st.from_addr) AS email,   
       st.due_date AS due_date,
       st.priority AS priority_type,
       st.assigned_to AS ticket_assigned_to,
       assignee.name AS ticket_assigned_name,
       f.submitted_at AS form_submitted_at,
       st.tickets_received AS tickets_received_at,
       f.payment_task AS payment_task,
       cfs.closed_by AS closed_by_id,
       cs.name AS form_closed_by,
       cfs.submitted_at AS form_closed_at,
       CASE 
           WHEN cfs.status = 'approve' THEN 'Completed'
           WHEN cfs.status = 'reject' THEN 'Rejected'
           WHEN cfs.status = 'request' THEN 'Waiting for user information/waiting for user action'
           WHEN cfs.status IS NULL THEN 'Not Yet closed/Need Admin Action/Admin Action needed'
           ELSE 'Unknown'
       END AS form_status,
       st.subject AS subject,
       st.body AS body
FROM comm_tbl_smthub_ticket st
LEFT JOIN (
    SELECT ac.form_id, ac.userid, ac.username, ac.email, ac.submitted_at, ac.payment_created_at AS payment_task, 1 AS tag_id
    FROM comm_account_closure ac
    UNION ALL
    SELECT cm.id AS form_id, cm.userid, cm.username, cm.email, cm.submitted_at, NULL AS payment_task, 2 AS tag_id
    FROM comm_content_mapping cm
    UNION ALL
    SELECT rf.id AS form_id, rf.userid, rf.username, rf.email, rf.submitted_at, NULL AS payment_task, 4 AS tag_id
    FROM comm_revenue_form rf
    UNION ALL
    SELECT oac.id AS form_id, oac.userid, oac.username, oac.email, oac.submitted_at, NULL AS payment_task, 3 AS tag_id
    FROM comm_oac_form oac
) f ON st.id = f.form_id AND st.tag_id = f.tag_id
LEFT JOIN comm_form_status cfs ON cfs.form_id = f.form_id
LEFT JOIN comm_staffs cs ON cfs.closed_by = cs.uid
LEFT JOIN comm_staffs assignee ON st.assigned_to = assignee.uid
WHERE st.tag_id IN (1, 2, 3, 4);


---If they ask upc in forms/upc related to any forms, generate below query--- 

example:
SELECT 
    'revenue form' AS form_name,
    rf.id, 
    rf.userid, 
    u.name AS username, 
    u.mail AS email, 
    rfm.RevenueShareTo, 
    rf.submitted_at 
FROM comm_revenue_form rf
JOIN users u ON rf.userid = u.uid
JOIN comm_revenue_form_multiple rfm ON rf.id = rfm.Sid
WHERE rfm.RevenueShareTo LIKE '%<upc>%'

UNION ALL 

SELECT 
    'Content form' AS form_name,
    cm.id, 
    cm.userid, 
    u.name AS username, 
    u.mail AS email, 
    cm.submitted_at, 
    csd.upc 
FROM comm_content_mapping cm
JOIN users u ON cm.userid = u.uid
JOIN comm_store_details csd ON csd.tbl_content_mapping_id = cm.id
WHERE csd.upc = '<upc>'

UNION ALL

SELECT 
    'oac form' AS form_name,
    oc.id, 
    oc.userid, 
    u.name AS username, 
    u.mail AS email, 
    oc.upc, 
    oc.submitted_at 
FROM comm_oac_form oc
JOIN users u ON oc.userid = u.uid
WHERE oc.upc LIKE '%<upc>%';


Forms and tickets have relationship using 
    comm_oac_form.keys1 == comm_tbl_smthub_ticket.keys1
    comm_revenue_form.keys1 == comm_tbl_smthub_ticket.keys1
    comm_account_closure.keys1 == comm_tbl_smthub_ticket.keys1
    comm_content_mapping.keys1 == comm_tbl_smthub_ticket.keys1
    comm_oac_form.id == comm_tbl_smthub_ticket.id
    comm_revenue_form.id == comm_tbl_smthub_ticket.id
    comm_account_closure.form_id == comm_tbl_smthub_ticket.id
    comm_content_mapping.id == comm_tbl_smthub_ticket.id


So we check status of tickets after it's move to form check the status in comm_form_status.

--For Tickets/mails/inbox/ticket related question use this Query:--
sample questions : what is the status of tickets in account closure?

can't determine tag_name ,don't pass that in where condition use below query without change.

tickets have priority column having values Easy,Moderate,High and Very High
availabe tag_name are 
1 Account Closures 
2 	Content Mapping 
3 	YouTube 	
4 	Revenue Share 	
6 	403 Error 
7 	Allow List 
8 	Artwork 		
9 	Audio Issue 		
10 	Bank Transfer 		
12 	Copyright Infringement 		
13 	Facebook/Instagram Verification 		
14 	Facebook/Instagram/CID 		
15 	Failed Upload/ Failed Takedown 		
16 	Mod Delay 		
17 	New Feature Feedback 		
18 	Password 		
19 	Payments & Pay Updates 		
20 	Refund 		
21 	RN Advance 		
22 	Stats & Reports 		
23 	Takedown or Closure 		
24 	Disapprove all pending releases 		
25 	Delete all unfinished releases 		
26 	Adds a note to all RFS releases 	
27 	Adds a flag to the account 
28 	Takedown for all stores 	
30 	Manual Account Activation 	
31 	RFS all active releases 	
32 	PayPal


availabe action names are
1 	Account Closure 
2 	Content Mapping 	
3 	Youtube OAC 
4 	Revenue Share
5 	Disapprove all pending releases 
6 	Delete all unfinished releases 
7 	Adds a note to all RFS releases 
8 	Adds a flag to the account 
9 	RFS all active releases 
10 	Takedown for all stores 
11 	Unlinks all YT channels 
12 	Manual Account Activation 
14 	Close Thread
 

if they ask 'all mails' list all tickets 
if 'user based tickets' where comm_tbl_smthub_ticket.upc_id is null or upc_id '' 
else if 'upc based tickets' where comm_tbl_smthub_ticket.upc_id not null 
else if 'uncategorised tickets' where comm_tbl_smthub_ticket.tag_id not null 
else if 'inbox tickets' where comm_tbl_smthub_ticket.tag_id in 1,2,3,4 


if they ask about 
    account closure -- tt.id is 1
    revenue share -- tt.id is 4
    content mapping -- tt.id is 2
    youtube oac -- tt.id is 3
pass it in where condition and get it.

cgs.groups defines user is hp(high priority), priority, or premium users 

if they ask could you summarize ticket activity for this month, focusing on open, closed, and pending ones?
generate this

status column has values Received , Waiting For User Information, Admin Action Needed and Completed.

comm_tbl_smthub_ticket.upc_id  gives tickets related to any upc

SELECT 
    'TICKETS' AS 'FORM/TICKET',
    st.id AS ticket_id,
    tt.tag_name AS 'form_name/tag_name',
    aa.action_name AS action_name,
    st.uid AS from_userid,
    u.name AS from_username,
    st.from_addr AS from_email,
    cac.email AS submitted_email,
    cac.userID AS submitted_userid,
    cac.username AS submitted_username,
    st.due_date,st.priority,
    COALESCE(cg.me_email, 'No Associated Email') AS associated_email,
    COALESCE(cgs.groups, 'No Group') AS user_type, 
    st.upc_id AS UPC,
    NULL AS form_submitted_at,
    st.tickets_received AS tickets_received_at,
    NULL AS payment_task,
    COUNT(CASE WHEN st.completed_at IS NOT NULL AND st.completed_at != '' THEN 1 END) AS completed_count,
    COUNT(CASE WHEN st.completed_at IS NULL OR st.completed_at = '' THEN 1 END) AS not_completed_count,
    COALESCE(st.assigned_to, 'N/A') AS tickets_closed_by_id,
    COALESCE(cs.name, 'Not Assigned') AS tickets_closed_by,
    NULL AS form_closed_at,
    st.completed_at AS ticket_closed_at,
     st.instant_action AS instant_action,
     st.smart_schedule AS smart_schedule,
    st.status AS 'ticket_status/form_status',
    st.subject,
    st.body
FROM comm_tbl_smthub_ticket st
LEFT JOIN comm_tag tt ON tt.id = st.tag_id
LEFT JOIN comm_action aa ON aa.id = st.action_id
LEFT JOIN comm_account_closure cac ON cac.form_id = st.id
LEFT JOIN users u ON st.uid = u.uid
LEFT JOIN comm_staffs cs ON cs.uid = st.assigned_to
LEFT JOIN comm_grouplist cg ON cg.userid = u.uid
LEFT JOIN comm_groups cgs ON cgs.id = cg.groupid
GROUP BY  cac.email,cac.userID,cac.username,st.id, st.status,st.instant_action,st.smart_schedule,st.due_date,st.upc_id,st.priority, st.uid, tt.tag_name,aa.action_name, st.tickets_received, st.completed_at, st.assigned_to, u.name, st.from_addr, cs.name, st.subject, st.body, cg.me_email, cgs.groups;



Use LEFT JOIN if they don't mention type of form  else get only select query without LEFT JOIN all.


Ensure only the {question} section is dynamically adjusted based on the input. 

{question} 


       """
     
        print(generate_sql(prompt))
    else:
        print("Provide input as command-line argument")








